Easy to use performance tracer for Ruby

## Installation

Add this line to your application's Gemfile:

```ruby
gem 'owl_tracer'
```

## Usage

Step 1, instantiate the tracer:
```
tracer = OwlTracer.new
```

Step 2, run whatever code you want to benchmark. Let's say we run two commands:
```
review_presenter.prepare
stats_calcularor.calc
```

Step 3, get the results:
```
pp tracer.dump_results(100) # Only calls that took at least 100ms will be reported
```
and you'll receive an array of strings with all your application method calls with how many milliseconds it took to run, properly idented. It will look like this:
```
ReviewPresenter::prepare (1273 ms)
  ChildPresenter::initialize (1047 ms)
    ChildPresenter::initialize_pricing_instance_variables (260 ms)
      ChildPresenter::initialize_quoted_rate_presenter (203 ms)
    IncomePresenter::initialize_income_presenter (260 ms)
      JobPresenter::initialize (238 ms)
        Review::JobVerifier::initialize (238 ms)
          Review::JobVerifier::load_jobs (130 ms)
StatsCalculator::calc (118 ms)
```

You can also skip the number of milliseconds or pass 0 and this will produce a very verbose list of all method calls that happened between the tracer was instantiated until you asked for the dump.
The first time you call `dump_results`, it stops recording new calls, but you can call `dump_results` multiple times with different numbers to zoom in or out what was recorded.

## Web Usage
To figure out why a route is slow in your web app, just add this to your app's `application_controller`:

```
class ApplicationController < ActionController::Base
  ...
  around_action :owl_tracer_logger
  ...
  
    def global_request_logging
      # if a param comes in the request, we start the tracer
      if params[:owl_tracer] 
        tracer = OwlTracer.new
      end  

      # we let the request be hadled normally
      yield
  
      if params[:owl_tracer]
        # get the results
        trace_result = tracer.dump_results(params[:owl_tracer].to_i)
        
        # them write it to a log file
        File.write("/tmp/owl_tracer.txt", trace_result.join("\n"))
        
        # or print it in the logs
        warn(trace_result)
        
        # or email it to you... or save it on S3... etc.
      end
  end
```

after that, you can add `?owl_tracer=200` (or `&owl_tracer=200`) to any route, like for example  `http://your_server/whatever_endpoint_you_want?owl_tracer=200` to get everything that is taking more than `200ms` , including controller time, rendering time and code triggered by the view time, like:

```
SlowController::show (1073 ms)
  PastPresenter::initialize (1047 ms)
    PastPresenter::initialize_pricing_instance_variables (260 ms)
      PastPresenter::initialize_quoted_rate_PastPresenter (203 ms)
    PastPresenter::initialize_income_PastPresenter (260 ms)
      JobPastPresenter::initialize (238 ms)
        Slow::JobVerifier::initialize (238 ms)
show.html.slim::rendering (2474 ms)
  _summary.html.slim::rendering (1842 ms)
    _collapse_panels.html.slim::rendering (549 ms)
      _Education_panel.html.slim::rendering (389 ms)
        EducationPresenter::data_json (386 ms)
          EducationPresenter::data_hash (345 ms)
            EducationPresenter::verifications (306 ms)
    sHelper::_supportables (225 ms)
      #<Class:SupportingDocs::SupportableServicer>::get_all_doc_supportables (225 ms)
  _tabs.html.slim::rendering (558 ms)
    _info_tab.html.slim::rendering (427 ms)
      _education_info.html.slim::rendering (261 ms)
        _bank_info.html.slim::rendering (217 ms)
```

making it easier to identify why the route is slow.

